@@ -4,6 +4,22 @@ import pandas as pd
 import os
 import json
 from urllib.parse import urlparse
+from typing import List
+
+# Async
+import asyncio
+from functools import wraps, partial
+
+
+def async_wrap(func):
+    @wraps(func)
+    async def run(*args, loop=None, executor=None, **kwargs):
+        if loop is None:
+            loop = asyncio.get_event_loop()
+        pfunc = partial(func, *args, **kwargs)
+        return await loop.run_in_executor(executor, pfunc)
+    return run 
+
 
 class Vv:
     def __init__(self):
@@ -159,9 +175,29 @@ class Vv:
     
     def retrieve_picklist_values(self, picklist_name):
         """
-        Note: The picklist_name may not be the same as the picklist field's API name.
+        Note: This is not the picklist field's API name, but the picklist (to which the picklist field looks up to) API name.
+        For example, the picklist field "specialty_1__v", "specialty_2__v" and "specialty_3__v" all look up to the picklist "specialty__v".
         """
         url = f"{self.vaultURL}/api/{self.LatestAPIversion}/objects/picklists/{picklist_name}"
-        r = requests.get(url, headers = self.APIheaders).json()['picklistValues']
-        return pd.DataFrame(r)
+        r = requests.get(url, headers = self.APIheaders).json()
+        if r['responseStatus'] == 'SUCCESS':
+            result = pd.DataFrame(r['picklistValues'])
+            result['picklist_api_name'] = picklist_name
+            return result
+        else:
+            raise Exception(r['errors'][0]['type'] + ": " + r['errors'][0]['message'])
+    
+    async def async_bulk_retrieve_picklist_values(self, queries: List[str]) -> pd.DataFrame:
+        """_summary_: This function is the async version of the retrieve_picklist_values function. It is used to retrieve multiple picklist values in parallel.
+
+        Args:
+            queries (List[str]): List of picklist API names
+
+        Returns:
+            _type_: pd.DataFrame
+        """
+        async_queries = async_wrap(self.retrieve_picklist_values)
+        result_list = await asyncio.gather(*[async_queries(query) for query in queries])
+        result_pd = pd.concat(result_list, ignore_index=True)
+        return result_pd
         
\ No newline at end of file
